--Lua script for Scrap 'n' Shake
--Player is initially given a force of offensive and defensive units
--and has to secure an area for the incoming Recycler, and secure a scrap field
--Once the Recycler reaches the player, the player must start collecting
--all the scrap he can!

--Game Objects
local Player = nil
local PlayerRecycler = nil
local RecyDefenderOne = nil
local RecyDefenderTwo = nil
local RecyDefenderThree = nil

local PlayerStartOffenseTable = {"avtank", "avfigh", "avrckt", "avltnk"}
local PlayerStartDefenseTable = {"avturr", "avartl", "avmine"}

--Game Objectives
local ScrapNeeded = 200
local RecyclerArrived = false

--Game Data
local isTremoring = false
local tremorMagnitude = 0
local tremorTime = 0		--seconds
local tremorBasePeriod = 30	--base time between tremors
local tremorPeriod = 0

	--Audio
	local AudioWelcomeMessage = ""
	local AudioSuccessMessage = ""
	local AudioLostRecycler = ""
	local AudioTremorIncoming = ""

	--Objectives
	local ObjectiveScrap = "objscrap.otf"
	local ObjectiveWaitForRecycler = "objwrecy.otf"
	local ObjectiveRecycler = "objrecya.otf"

	--Briefings
	local BriefSuccess = "successI.des"
	local BriefFailure = "failureI.des"

local PilotsAdded = 5 --Pilots added after each tremor

local PlayerOffensiveCountStart = 5
local PlayerDefensiveCountStart = 3

--PathSpawns
local CCASpawn = "path_2"
local PlayerRecyclerSpawn = "path_1"

--Script Data
local startDone = false
local AIPFile = "scrapfun.aip"
local firstRun = true
local objectiveShown = false

function CreateTremor()
	tremorMagnitude = math.random(1, 10)
	tremorTime = math.random(5, 45)
	tremorPeriod = tremorBasePeriod * math.random(0.5, 1.5)
end

function GiveResourcesToAI()
	SetPilot(2, 10)
	SetScrap(2, 30)
end

function Update()

	Player = GetPlayerHandle()

	--Initial setup
	if not startDone then
		SetAIControl(2, true)
		SetAIP(AIPFile, 2)

		--Spawn Player's starting units
		--offensive
		for i = 1, PlayerOffensiveCountStart do
			BuildObject(PlayerStartOffenseTable[math.random(# PlayerStartOffenseTable)], 1, "path_"..tostring(i + 3))
		end

		--defensive
		for i = 1, PlayerDefensiveCountStart do
			BuildObject(PlayerStartDefenseTable[math.random(# PlayerStartDefenseTable)], 1, "path_"..tostring(i + 8))
		end

		--Recycler starts heading to the player with defenders
		PlayerRecycler = BuildObject("avrecy", 1, PlayerRecyclerSpawn)
		RecyDefenderOne = BuildObject("avtank", 1, PlayerRecycler)
		RecyDefenderTwo = BuildObject("avfigh", 1, PlayerRecycler)
		RecyDefenderThree = BuildObject("avfigh", 1, PlayerRecycler)

		Goto(PlayerRecycler, Player, 1)
		Follow(RecyDefenderOne, PlayerRecycler, 1)
		Follow(RecyDefenderTwo, PlayerRecycler, 1)
		Follow(RecyDefenderThree, PlayerRecycler, 1)

		--Welcome player!
		--AudioMessage(AudioWelcomeMessage)
		--ClearObjectives()
		--AddObjective(ObjectiveWaitForRecycler)

		startDone = true
	end

	if not firstRun and not objectiveShown then
		AddObjective(ObjectiveWaitForRecycler)
		objectiveShown = true
	end

	--Has the recycler arrived yet and is it close to the player?
	if not RecyclerArrived and IsWithin(PlayerRecycler, Player, 100) then
		--Recycler stops and waits for commands
		--it's defenders follow the player now, and are now commandable
		Stop(PlayerRecycler, 0)
		Follow(RecyDefenderOne, Player, 0)
		Follow(RecyDefenderTwo, Player, 0)
		Follow(RecyDefenderThree, Player, 0)

		ClearObjectives()
		AddObjective(ObjectiveScrap)
		AddObjective(ObjectiveRecycler)

		RecyclerArrived = true
	else
		Goto(PlayerRecycler, Player, 1)
	end


	--Tremor conditions

	--is there no tremor right now? And has the waiting period expired?
	if not isTremoring and GetCockpitTimer() == 0 then
		CreateTremor()

		if math.random(0, 1) == 1 then
			--AudioMessage(AudioTremorIncoming)

			--wait for warning to end before starting the tremor
			while IsAudioMessagePlaying() do end
		end

		StartEarthquake(tremorMagnitude)
		StartCockpitTimer(tremorTime)
		HideCockpitTimer()

		isTremoring = true
	end

	--is there a tremor going on? Has it's timer run out?
	if isTremoring and GetCockpitTimer() == 0 then
		StopEarthquake()
		isTremoring = false

		StartCockpitTimer(tremorPeriod)
		HideCockpitTimer()
		AddPilot(1, PilotsAdded)
		GiveResourcesToAI()
	end


	--Victory/Loss conditions

	--player lost his recycler
	if not IsValid(PlayerRecycler) then
		--AudioMessage(AudioPlayerLostRecycler)

		ClearObjectives()
		AddObjective(ObjectiveScrap)
		AddObjective(ObjectiveRecycler, "red")

		FailMission(8, BriefFailure)
	end

	--player gets the required scrap!
	if GetScrap(1) == ScrapNeeded then
		--AudioMessage(AudioSuccessMessage)

		ClearObjectives()
		AddObjective(ObjectiveScrap, "green")
		AddObjective(ObjectiveRecycler, "green")

		SucceedMission(8, BriefSuccess)
	end

	if firstRun then
		firstRun = false
	end

end
